@@ -1,4 +1,5 @@
 # Copyright 2015 Futurewei.  All rights reserved.
+# Copyright 2017 Intel Corporation.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -29,10 +30,13 @@ from neutron.db import common_db_mixin
 from neutron.db import models_v2
 from neutron_lib.db import constants as db_const
 from neutron_lib.db import model_base
+from neutron_lib.db import utils as db_utils
 
 from networking_sfc.db import flowclassifier_db as fc_db
 from networking_sfc.extensions import flowclassifier as ext_fc
+from networking_sfc.extensions import servicegraph as ext_sg
 from networking_sfc.extensions import sfc as ext_sfc
+from networking_sfc.extensions import tap as ext_tap
 
 
 LOG = logging.getLogger(__name__)
@@ -147,6 +151,8 @@ class PortPairGroup(model_base.BASEV2, model_base.HasId,
     group_id = sa.Column(sa.Integer(), unique=True, nullable=False)
     name = sa.Column(sa.String(db_const.NAME_FIELD_SIZE))
     description = sa.Column(sa.String(db_const.DESCRIPTION_FIELD_SIZE))
+    tap_enabled = sa.Column(sa.Boolean(), server_default=sa.sql.false(),
+                            nullable=False)
     port_pairs = orm.relationship(
         PortPair,
         backref='port_pair_group'
@@ -182,9 +188,38 @@ class PortChain(model_base.BASEV2, model_base.HasId, model_base.HasProject):
         cascade='all, delete-orphan')
 
 
+class GraphChainAssoc(model_base.BASEV2):
+    """Relation table between Service Graphs and src+dst Port Chains."""
+    __tablename__ = 'sfc_service_graph_chain_associations'
+    service_graph_id = sa.Column(
+        sa.String(UUID_LEN),
+        sa.ForeignKey('sfc_service_graphs.id', ondelete='CASCADE'),
+        primary_key=True, nullable=False)
+    src_chain = sa.Column(
+        sa.String(UUID_LEN),
+        sa.ForeignKey('sfc_port_chains.id', ondelete='RESTRICT'),
+        primary_key=True, nullable=False)
+    dst_chain = sa.Column(
+        sa.String(UUID_LEN),
+        sa.ForeignKey('sfc_port_chains.id', ondelete='RESTRICT'),
+        primary_key=True, nullable=False)
+
+
+class ServiceGraph(model_base.BASEV2, model_base.HasId, model_base.HasProject):
+    """Represents a Neutron Service Function Chain Graph."""
+    __tablename__ = 'sfc_service_graphs'
+    name = sa.Column(sa.String(db_const.NAME_FIELD_SIZE))
+    description = sa.Column(sa.String(db_const.DESCRIPTION_FIELD_SIZE))
+    graph_chain_associations = orm.relationship(
+        GraphChainAssoc,
+        backref='service_graph',
+        cascade='all, delete-orphan')
+
+
 class SfcDbPlugin(
     ext_sfc.SfcPluginBase,
-    common_db_mixin.CommonDbMixin
+    common_db_mixin.CommonDbMixin,
+    ext_sg.ServiceGraphPluginBase
 ):
     """Mixin class to add port chain to db_plugin_base_v2."""
 
@@ -212,8 +247,14 @@ class SfcDbPlugin(
 
     def _validate_port_pair_groups(self, context, pg_ids, pc_id=None):
         with db_api.context_manager.reader.using(context):
+            prev_pg_tap_enabled = False
             for pg_id in pg_ids:
-                self._get_port_pair_group(context, pg_id)
+                pg = self._get_port_pair_group(context, pg_id)
+                curr_pg_tap_enabled = pg['tap_enabled']
+                if prev_pg_tap_enabled and curr_pg_tap_enabled:
+                    raise ext_tap.ConsecutiveTapPPGNotSupported()
+                else:
+                    prev_pg_tap_enabled = curr_pg_tap_enabled
             query = self._model_query(context, PortChain)
             for port_chain_db in query.all():
                 if port_chain_db['id'] == pc_id:
@@ -226,6 +267,18 @@ class SfcDbPlugin(
                     raise ext_sfc.InvalidPortPairGroups(
                         port_pair_groups=pg_ids, port_chain=port_chain_db.id)
 
+    def _validate_correlation_consistency(self, context, ppg_ids, pc_corr):
+        # format like in ServiceFunctionParam.value to aid comparison later:
+        pc_corr = jsonutils.dumps(pc_corr)
+        with db_api.context_manager.reader.using(context):
+            for ppg_id in ppg_ids:
+                ppg = self._get_port_pair_group(context, ppg_id)
+                for pp in ppg['port_pairs']:
+                    pp_corr = pp['service_function_parameters']['correlation']
+                    if pp_corr.value != 'null' and pp_corr.value != pc_corr:
+                        raise ext_sfc.PortChainInconsistentCorrelations(
+                            ppg=ppg_id)
+
     def _validate_flow_classifiers(self, context, fc_ids, pc_id=None):
         with db_api.context_manager.reader.using(context):
             fcs = [
@@ -307,9 +360,11 @@ class SfcDbPlugin(
                 key: ChainParameter(keyword=key, value=jsonutils.dumps(val))
                 for key, val in pc['chain_parameters'].items()}
 
-            pg_ids = pc['port_pair_groups']
+            ppg_ids = pc['port_pair_groups']
             fc_ids = pc['flow_classifiers']
-            self._validate_port_pair_groups(context, pg_ids)
+            self._validate_port_pair_groups(context, ppg_ids)
+            self._validate_correlation_consistency(
+                context, ppg_ids, pc['chain_parameters']['correlation'])
             self._validate_flow_classifiers(context, fc_ids)
             assigned_chain_ids = {}
             query = context.session.query(PortChain)
@@ -337,7 +392,7 @@ class SfcDbPlugin(
                                       chain_parameters=chain_parameters,
                                       chain_id=chain_id)
             self._setup_chain_group_associations(
-                context, port_chain_db, pg_ids)
+                context, port_chain_db, ppg_ids)
             self._setup_chain_classifier_associations(
                 context, port_chain_db, fc_ids)
             context.session.add(port_chain_db)
@@ -348,8 +403,8 @@ class SfcDbPlugin(
     def get_port_chains(self, context, filters=None, fields=None,
                         sorts=None, limit=None,
                         marker=None, page_reverse=False, default_sg=False):
-
-        marker_obj = self._get_marker_obj(context, 'port_chain', limit, marker)
+        marker_obj = db_utils.get_marker_obj(self, context, 'port_chain',
+                                             limit, marker)
         return self._get_collection(context,
                                     PortChain,
                                     self._make_port_chain_dict,
@@ -376,6 +431,9 @@ class SfcDbPlugin(
 
     @log_helpers.log_method_call
     def delete_port_chain(self, context, id):
+        # if this port chain is part of a graph, abort delete:
+        if self._any_port_chains_in_a_graph(context, {id}):
+            raise ext_sg.ServiceGraphPortChainInUse(id=id)
         try:
             with db_api.context_manager.writer.using(context):
                 pc = self._get_port_chain(context, id)
@@ -386,6 +444,11 @@ class SfcDbPlugin(
     @log_helpers.log_method_call
     def update_port_chain(self, context, id, port_chain):
         pc = port_chain['port_chain']
+        # if this port chain is part of a graph, abort non-neutral updates:
+        if self._any_port_chains_in_a_graph(context, {id}):
+            if 'flow_classifiers' in pc or 'port_pair_groups' in pc:
+                raise ext_sg.ServiceGraphPortChainInUse(id=id)
+
         with db_api.context_manager.writer.using(context):
             pc_db = self._get_port_chain(context, id)
             for k, v in pc.items():
@@ -476,8 +539,8 @@ class SfcDbPlugin(
     def get_port_pairs(self, context, filters=None, fields=None,
                        sorts=None, limit=None, marker=None,
                        page_reverse=False):
-        marker_obj = self._get_marker_obj(context, 'port_pair',
-                                          limit, marker)
+        marker_obj = db_utils.get_marker_obj(self, context, 'port_pair',
+                                             limit, marker)
         return self._get_collection(context,
                                     PortPair,
                                     self._make_port_pair_dict,
@@ -533,6 +596,7 @@ class SfcDbPlugin(
             'description': port_pair_group['description'],
             'project_id': port_pair_group['project_id'],
             'port_pairs': [pp['id'] for pp in port_pair_group['port_pairs']],
+            'tap_enabled': port_pair_group['tap_enabled'],
             'port_pair_group_parameters': {
                 param['keyword']: jsonutils.loads(param['value'])
                 for k, param in
@@ -572,6 +636,8 @@ class SfcDbPlugin(
             portpairs_list = [self._get_port_pair(context, pp_id)
                               for pp_id in pg['port_pairs']]
             self._validate_pps_in_ppg(portpairs_list)
+            if pg['tap_enabled'] and len(portpairs_list) > 1:
+                raise ext_tap.MultiplePortPairsInTapPPGNotSupported()
             port_pair_group_parameters = {
                 key: PortPairGroupParam(
                     keyword=key, value=jsonutils.dumps(val))
@@ -598,7 +664,9 @@ class SfcDbPlugin(
                 project_id=project_id,
                 port_pairs=portpairs_list,
                 port_pair_group_parameters=port_pair_group_parameters,
-                group_id=group_id)
+                group_id=group_id,
+                tap_enabled=pg['tap_enabled']
+            )
             context.session.add(port_pair_group_db)
             return self._make_port_pair_group_dict(port_pair_group_db)
 
@@ -606,8 +674,8 @@ class SfcDbPlugin(
     def get_port_pair_groups(self, context, filters=None, fields=None,
                              sorts=None, limit=None, marker=None,
                              page_reverse=False):
-        marker_obj = self._get_marker_obj(context, 'port_pair_group',
-                                          limit, marker)
+        marker_obj = db_utils.get_marker_obj(self, context, 'port_pair_group',
+                                             limit, marker)
         return self._get_collection(context,
                                     PortPairGroup,
                                     self._make_port_pair_group_dict,
@@ -646,6 +714,8 @@ class SfcDbPlugin(
                               for pp_id in new_pg.get('port_pairs', [])]
             self._validate_pps_in_ppg(portpairs_list, id)
             old_pg = self._get_port_pair_group(context, id)
+            if old_pg['tap_enabled'] and len(portpairs_list) > 1:
+                raise ext_tap.MultiplePortPairsInTapPPGNotSupported()
             for k, v in new_pg.items():
                 if k == 'port_pairs':
                     port_pairs = [
@@ -668,3 +738,245 @@ class SfcDbPlugin(
                 context.session.delete(pg)
         except ext_sfc.PortPairGroupNotFound:
             LOG.info("Deleting a non-existing port pair group.")
+
+    def _graph_assocs_to_pc_dict(self, assocs):
+        assoc_dict = {}
+        for assoc in assocs:
+            if assoc.src_chain in assoc_dict:
+                assoc_dict[assoc.src_chain].append(assoc.dst_chain)
+            else:
+                assoc_dict[assoc.src_chain] = [assoc.dst_chain]
+        return assoc_dict
+
+    def _make_service_graph_dict(self, graph_db, fields=None):
+        res = {
+            'id': graph_db['id'],
+            'name': graph_db['name'],
+            'project_id': graph_db['project_id'],
+            'description': graph_db['description'],
+            'port_chains': self._graph_assocs_to_pc_dict(
+                graph_db['graph_chain_associations'])
+        }
+        return self._fields(res, fields)
+
+    def _make_graph_chain_assoc_dict(self, assoc_db, fields=None):
+        res = {
+            'service_graph_id': assoc_db['service_graph_id'],
+            'src_chain': assoc_db['src_chain'],
+            'dst_chain': assoc_db['dst_chain']
+        }
+        return self._fields(res, fields)
+
+    def _is_there_a_loop(self, parenthood, current_chain, traversed):
+        if current_chain not in parenthood:
+            return False
+        if current_chain not in traversed:
+            loop_status = False
+            traversed.append(current_chain)
+            for port_chain in parenthood[current_chain]:
+                loop_status = loop_status or self._is_there_a_loop(
+                    parenthood, port_chain, list(traversed))
+            return loop_status
+        return True
+
+    def _any_port_chains_in_a_graph(self, context,
+                                    port_chains=set(), graph_id=None):
+        if not port_chains:
+            return False
+        with db_api.context_manager.reader.using(context):
+            query = self._model_query(context, ServiceGraph)
+            for graph_db in query.all():
+                if graph_db['id'] == graph_id:
+                    continue
+                pc_ids = [
+                    assoc['src_chain']
+                    for assoc in graph_db.graph_chain_associations
+                ]
+                pc_ids.extend([
+                    assoc['dst_chain']
+                    for assoc in graph_db.graph_chain_associations
+                ])
+                if pc_ids and port_chains and set(
+                        pc_ids).intersection(port_chains):
+                    return True
+        return False
+
+    def _validate_port_chains_for_graph(self, context,
+                                        port_chains, graph_id=None):
+            # create a list of all port-chains that will be associated
+            all_port_chains = set()
+            for src_chain in port_chains:
+                all_port_chains.add(src_chain)
+                for dst_chain in port_chains[src_chain]:
+                    all_port_chains.add(dst_chain)
+            # check if any of the port-chains are already in a graph
+            if self._any_port_chains_in_a_graph(
+                    context, all_port_chains, graph_id):
+                raise ext_sg.ServiceGraphInvalidPortChains(
+                    port_chains=port_chains)
+
+            # dict whose keys are PCs and values are lists of dependency-PCs
+            # (PCs incoming to the point where the key is a outgoing)
+            parenthood = {}
+            encapsulation = None
+            fc_cls = fc_db.FlowClassifierDbPlugin
+            for src_chain in port_chains:
+                src_pc = self._get_port_chain(context, src_chain)
+                curr_corr = src_pc.chain_parameters['correlation']['value']
+                # guarantee that branching PPG supports correlation
+                assocs = src_pc.chain_group_associations
+                src_ppg = max(assocs, key=(lambda ppg: ppg.position))
+                ppg_id = src_ppg['portpairgroup_id']
+                ppg = self._get_port_pair_group(context, ppg_id)
+                for pp in ppg.port_pairs:
+                    sfparams = pp['service_function_parameters']
+                    if sfparams['correlation']['value'] != curr_corr:
+                        raise ext_sg.ServiceGraphImpossibleBranching()
+
+                # verify encapsulation consistency across all PCs (part 1)
+                if not encapsulation:
+                    encapsulation = curr_corr
+                elif encapsulation != curr_corr:
+                    raise ext_sg.ServiceGraphInconsistentEncapsulation()
+                # list of all port chains at this branching point:
+                branching_point = []
+                # list of every flow classifier at this branching point:
+                fcs_for_src_chain = []
+                for dst_chain in port_chains[src_chain]:
+                    # check if the current destination PC was already added
+                    if dst_chain in branching_point:
+                        raise ext_sg.ServiceGraphPortChainInConflict(
+                            pc_id=dst_chain)
+                    branching_point.append(dst_chain)
+                    dst_pc = self._get_port_chain(context, dst_chain)
+                    curr_corr = dst_pc.chain_parameters['correlation']['value']
+                    # guarantee that destination PPG supports correlation
+                    assocs = dst_pc.chain_group_associations
+                    dst_ppg = min(assocs, key=(lambda ppg: ppg.position))
+                    ppg_id = dst_ppg['portpairgroup_id']
+                    ppg = self._get_port_pair_group(context, ppg_id)
+                    for pp in ppg.port_pairs:
+                        sfparams = pp['service_function_parameters']
+                        if sfparams['correlation']['value'] != curr_corr:
+                            raise ext_sg.ServiceGraphImpossibleBranching()
+                    # verify encapsulation consistency across all PCs (part 2)
+                    if encapsulation != curr_corr:
+                        raise ext_sg.ServiceGraphInconsistentEncapsulation()
+                    dst_pc_dict = self._make_port_chain_dict(dst_pc)
+                    # acquire associated flow classifiers
+                    fcs = dst_pc_dict['flow_classifiers']
+                    for fc_id in fcs:
+                        fc = self._get_flow_classifier(context, fc_id)
+                        fcs_for_src_chain.append(fc)  # update list of every FC
+                    # update the parenthood dict
+                    if dst_chain in parenthood:
+                        parenthood[dst_chain].append(src_chain)
+                    else:
+                        parenthood[dst_chain] = [src_chain]
+                    # detect duplicate FCs, consequently branching ambiguity
+                    for i, fc1 in enumerate(fcs_for_src_chain):
+                        for fc2 in fcs_for_src_chain[i + 1:]:
+                            if(fc_cls.flowclassifier_basic_conflict(fc1, fc2)):
+                                raise ext_sg.\
+                                    ServiceGraphFlowClassifierInConflict(
+                                        fc1_id=fc1['id'], fc2_id=fc2['id'])
+
+            # check for circular paths within the graph via parenthood dict:
+            for port_chain in parenthood:
+                if self._is_there_a_loop(parenthood, port_chain, []):
+                    raise ext_sg.ServiceGraphLoopDetected()
+
+    def _setup_graph_chain_associations(self, context, graph_db, port_chains):
+        with db_api.context_manager.reader.using(context):
+            graph_chain_associations = []
+            for src_chain in port_chains:
+                query = self._model_query(context, GraphChainAssoc)
+                for dst_chain in port_chains[src_chain]:
+                    graph_chain_association = query.filter_by(
+                        service_graph_id=graph_db.id,
+                        src_chain=src_chain, dst_chain=dst_chain).first()
+                    if not graph_chain_association:
+                        graph_chain_association = GraphChainAssoc(
+                            service_graph_id=graph_db.id,
+                            src_chain=src_chain,
+                            dst_chain=dst_chain
+                        )
+                    graph_chain_associations.append(graph_chain_association)
+            graph_db.graph_chain_associations = graph_chain_associations
+
+    def _get_branches(self, context, filters):
+        return self._get_collection(context,
+                                    GraphChainAssoc,
+                                    self._make_graph_chain_assoc_dict,
+                                    filters=filters)
+
+    @log_helpers.log_method_call
+    def create_service_graph(self, context, service_graph):
+        """Create a Service Graph."""
+        service_graph = service_graph['service_graph']
+        project_id = service_graph['project_id']
+        with db_api.context_manager.writer.using(context):
+            port_chains = service_graph['port_chains']
+            self._validate_port_chains_for_graph(context, port_chains)
+            graph_db = ServiceGraph(id=uuidutils.generate_uuid(),
+                                    project_id=project_id,
+                                    description=service_graph['description'],
+                                    name=service_graph['name'])
+            self._setup_graph_chain_associations(
+                context, graph_db, port_chains)
+            context.session.add(graph_db)
+            return self._make_service_graph_dict(graph_db)
+
+    @log_helpers.log_method_call
+    def get_service_graphs(self, context, filters=None,
+                           fields=None, sorts=None, limit=None,
+                           marker=None, page_reverse=False):
+        """Get Service Graphs."""
+        marker_obj = db_utils.get_marker_obj(self, context,
+                                             'service_graph', limit, marker)
+        return self._get_collection(context,
+                                    ServiceGraph,
+                                    self._make_service_graph_dict,
+                                    filters=filters, fields=fields,
+                                    sorts=sorts,
+                                    limit=limit, marker_obj=marker_obj,
+                                    page_reverse=page_reverse)
+
+    @log_helpers.log_method_call
+    def get_service_graph(self, context, id, fields=None):
+        """Get a Service Graph."""
+        service_graph = self._get_service_graph(context, id)
+        return self._make_service_graph_dict(service_graph, fields)
+
+    @log_helpers.log_method_call
+    def _get_service_graph(self, context, id):
+        try:
+            return self._get_by_id(context, ServiceGraph, id)
+        except exc.NoResultFound:
+            raise ext_sg.ServiceGraphNotFound(id=id)
+
+    @log_helpers.log_method_call
+    def update_service_graph(self, context, id, service_graph):
+        """Update a Service Graph."""
+        service_graph = service_graph['service_graph']
+        with db_api.context_manager.writer.using(context):
+            graph_db = self._get_service_graph(context, id)
+            for k, v in service_graph.items():
+                if k == 'port_chains':
+                    self._validate_port_chains_for_graph(
+                        context, v, graph_id=id)
+                    self._setup_graph_chain_associations(
+                        context, graph_db, v)
+                else:
+                    graph_db[k] = v
+            return self._make_service_graph_dict(graph_db)
+
+    @log_helpers.log_method_call
+    def delete_service_graph(self, context, id):
+        """Delete a Service Graph."""
+        try:
+            with db_api.context_manager.writer.using(context):
+                graph = self._get_service_graph(context, id)
+                context.session.delete(graph)
+        except ext_sfc.ServiceGraphNotFound:
+            LOG.info("Deleting a non-existing Service Graph.")
