@@ -12,33 +12,32 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import testscenarios
 import time
 
-from tempest.common import waiters
+from oslo_log import log
+from tempest.common import utils
 from tempest import config
 from tempest.lib.common.utils import test_utils
 from tempest.lib import decorators
-from tempest import test
 
 from networking_sfc.tests.tempest_plugin.tests.scenario import base
 
 CONF = config.CONF
-load_tests = testscenarios.load_tests_apply_scenarios
+LOG = log.getLogger(__name__)
 
 
 class TestSfc(base.SfcScenarioTest):
     @classmethod
     def skip_checks(cls):
         super(TestSfc, cls).skip_checks()
-        if not (CONF.network.project_networks_reachable
-                or CONF.network.public_network_id):
+        if not (CONF.network.project_networks_reachable or
+                CONF.network.public_network_id):
             msg = ('Either project_networks_reachable must be "true", or '
                    'public_network_id must be defined.')
             raise cls.skipException(msg)
         required_exts = ['sfc', 'flow_classifier']
         for ext in required_exts:
-            if not test.is_extension_enabled(ext, 'network'):
+            if not utils.is_extension_enabled(ext, 'network'):
                 msg = "%s Extension not enabled." % ext
                 raise cls.skipException(msg)
 
@@ -50,13 +49,19 @@ class TestSfc(base.SfcScenarioTest):
 
     def setUp(self):
         super(TestSfc, self).setUp()
+
+        self.multi_node = CONF.compute.min_compute_nodes > 1
+        if self.multi_node:
+            LOG.info("Running test on multi node")
+        else:
+            LOG.info("Running test on single node")
+        # Save servers UUIDs
+        self.servers = []
+
         self.ssh_user = CONF.validation.image_ssh_user
         self.keypair = self.create_keypair()
-        self.security_group = self._create_security_group()
-        self._create_security_group_rule(
-            self.security_group, protocol=None,
-            direction='ingress')
-        self.net1, self.subnet1, self.router1 = self.create_networks()
+        self.net1, self.subnet1, self.router1 = self.create_networks(
+            port_security_enabled=False)
         self.router2 = self._create_router()
         self.router3 = self._create_router()
         self.router2_net1 = self._create_port(self.net1['id'])
@@ -78,9 +83,9 @@ class TestSfc(base.SfcScenarioTest):
             self._get_server_port_id_and_ip4(server))
         return floating_ip, port_id, fixed_ip
 
-    def _create_floating_ip(self, server):
+    def _create_floating_ip(self, server, client=None):
         floating_ip = self.create_floating_ip(
-            server)
+            server, client=client)
         self.check_floating_ip_status(floating_ip, 'ACTIVE')
         floating_ip_addr = floating_ip['floating_ip_address']
         self.check_public_network_connectivity(
@@ -143,12 +148,12 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21a8')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_create_port_chain(self):
         self._create_port_chain_helper(False)
 
     @decorators.idempotent_id('35927961-1904-4a6b-9d08-ad819f1cf812')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_create_port_chain_symmetric(self):
         self._create_port_chain_helper(True)
 
@@ -229,12 +234,12 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21a9')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_create_port_chain_multi_flow_classifiers(self):
         self._create_port_chain_multi_fc_helper(False)
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21b1')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_create_port_chain_multi_flow_classifiers_symmetric(self):
         self._create_port_chain_multi_fc_helper(True)
 
@@ -293,12 +298,12 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21aa')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_create_port_chain_multi_port_pairs(self):
         self._create_port_chain_multi_port_pairs_helper(False)
 
-    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ad')
-    @test.services('compute', 'network')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f869be1e21ad')
+    @utils.services('compute', 'network')
     def test_create_port_chain_multi_port_pairs_symmetric(self):
         self._create_port_chain_multi_port_pairs_helper(True)
 
@@ -360,17 +365,17 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ab')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_create_port_chain_multi_port_pair_groups(self):
         self._create_port_chain_multi_ppg_helper(False)
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21b0')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_create_port_chain_multi_port_pair_groups_symmetric(self):
         self._create_port_chain_multi_ppg_helper(True)
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e22ab')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_create_multi_port_chain(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
@@ -441,7 +446,7 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ac')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_update_port_chain_add_flow_classifiers(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
@@ -505,7 +510,7 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ad')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_update_port_chain_remove_flow_classifiers(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
@@ -569,7 +574,7 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ae')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_update_port_chain_replace_flow_classifiers(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
@@ -633,7 +638,7 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21af')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_update_port_chain_add_port_pair_groups(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
@@ -693,7 +698,7 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21bf')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_update_port_chain_remove_port_pair_groups(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
@@ -755,7 +760,7 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21be')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_update_port_chain_replace_port_pair_groups(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
@@ -817,8 +822,8 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21bc')
-    @test.services('compute', 'network')
-    def test_update_port_chain_replace_port_pair_groups_flow_classifers(self):
+    @utils.services('compute', 'network')
+    def test_update_port_chain_replace_port_pair_groups_flow_classifiers(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
         ) = self._setup_server(self.net1)
@@ -895,7 +900,7 @@ class TestSfc(base.SfcScenarioTest):
         time.sleep(10)
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21bb')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_update_port_pair_group_add_port_pairs(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
@@ -949,7 +954,7 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ba')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_update_port_pair_group_remove_port_pairs(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
@@ -1000,7 +1005,7 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21b9')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_update_port_pair_group_replace_port_pairs(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
@@ -1051,7 +1056,7 @@ class TestSfc(base.SfcScenarioTest):
             private_key=self.keypair['private_key'])
 
     @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21b9')
-    @test.services('compute', 'network')
+    @utils.services('compute', 'network')
     def test_multi_port_chains_update_port_pair_group_replace_port_pairs(
         self
     ):
@@ -1163,18 +1168,32 @@ class TestSfc(base.SfcScenarioTest):
 
     def _create_server(self, network):
         kwargs = {}
-        if self.keypair is not None:
-            kwargs['key_name'] = self.keypair['name']
-        if self.security_group is not None:
-            kwargs['security_groups'] = [{'name': self.security_group['name']}]
-        server = self.create_server(
+        if self.multi_node:
+            kwargs["scheduler_hints"] = {'different_host': self.servers}
+
+        inst = self.create_server(
             networks=[{'uuid': network['id']}],
+            key_name=self.keypair['name'],
             wait_until='ACTIVE',
             **kwargs)
-        waiters.wait_for_server_status(self.servers_client,
-                                       server['id'], 'ACTIVE')
+
+        adm_get_server = self.os_admin.servers_client.show_server
+        server = adm_get_server(inst['id'])['server']
+
         self._check_tenant_network_connectivity(
             server, self.ssh_user, self.keypair['private_key'])
+
+        # Check server is on different node
+        if self.multi_node:
+            new_host = server["OS-EXT-SRV-ATTR:host"]
+            host_list = [adm_get_server(s)["server"]["OS-EXT-SRV-ATTR:host"]
+                         for s in self.servers]
+            self.assertNotIn(new_host, host_list,
+                             message="Failed to create servers on different "
+                                     "Compute nodes.")
+
+            self.servers.append(server["id"])
+
         return server
 
     def _add_router_interface(self, router_id, port_id):
